package org.jboss.rtgov.elasticsearch.processors;

import org.jboss.rtgov.elasticsearch.repo.ElasticSearchException;
import org.jboss.rtgov.elasticsearch.repo.elastic.ElasticRepo;
import org.jboss.rtgov.elasticsearch.repo.elastic.impl.api.BasicElasticAPIRepo;
import org.jboss.rtgov.elasticsearch.repo.elastic.impl.rest.BasicElasticRestRepo;
import org.mvel2.MVEL;
import org.overlord.rtgov.ep.EventProcessor;

import java.io.Serializable;
import java.util.UUID;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 * Created with IntelliJ IDEA.
 * User: imk@redhat.com
 * Date: 03/02/14
 * Time: 10:35
 */
public  class BasicElasticSearchProcessor extends EventProcessor {
    private static final Logger LOG=Logger.getLogger(BasicElasticSearchProcessor.class.getName());

    private static final String BASIC_ELASTIC_REPO = "BasicElasticRepo";
    private ElasticRepo basicElasticRestRepo = null;
    private String script=null;
    private Object scriptExpression=null;
    private String correleationScript = null ;

    public Object getScriptExpression() {
        return scriptExpression;
    }

    public void setScriptExpression(Object scriptExpression) {
        this.scriptExpression = scriptExpression;
    }

    public String getScript() {
        return script;
    }

    public void setScript(String script) {
        this.script = script;
    }

    @Override
    public void init() throws Exception {
        super.init();
        if(script != null && script.equals("NONE"))
            this.script="genrandom.mvel";

        // Load the script
        java.io.InputStream is=Thread.currentThread().getContextClassLoader().getResourceAsStream(script);

        if (is == null) {
            throw new Exception("Unable to locate MVEL script '"+script+"'");
        } else {
            byte[] b=new byte[is.available()];
            is.read(b);
            is.close();

            // Compile expression
            scriptExpression = MVEL.compileExpression(new String(b));

            if (LOG.isLoggable(Level.FINE)) {
                LOG.fine("Initialized script="+script
                        +" compiled="+scriptExpression);
            }
        }

        /**
         * expect type SimpleDocumentRepo;
         */
        basicElasticRestRepo = (BasicElasticAPIRepo) getServices().get(BASIC_ELASTIC_REPO);
    }

    @Override
    public  Serializable process(String source, Serializable event, int retriesLeft) throws Exception {
        return process(source,  event, retriesLeft, getRandom());
    }

    /**
     * @return
     */
    public  Serializable process(String source, Serializable event, int retriesLeft, String id) throws Exception{
        try {
            basicElasticRestRepo.persist(event,id);
        } catch (ElasticSearchException e) {
            e.printStackTrace();
            throw  e;
        }
        return event;
    }

    protected String getRandom() {
        return (UUID.randomUUID().toString());
    }
}
